module Goldie
  module Commands
    COMMAND_LIST = {
      add: "create a new item",
      update: "modify an item",
      done: "mark an item as done",
      not_done: "mark an item as not done",
      archive: "archive an item",
      unarchive: "unarchive an item",
      delete: "delete an item",
      list: "list certain items",
      progress: "show progress bar",
      list_progress: "list items and show progress bar",
      help: "list available commands"
    }

    extend self

    def add(title, weight = 1)
      item = Item.new(title: title, weight: weight)
      if item.save
        puts "Added item ##{item.id}."
        puts
        list_progress
      else
        puts "Item couldn't be saved:"
        _display_errors(item)
      end
    end

    def update(id, field, value)
      if item = _find_item(id)
        if %w(title weight weight_completed).include? field
          if item.update_attributes(field => value)
            puts "Updated item ##{item.id}."
            puts
            list_progress
          else
            puts "Item couldn't be updated:"
            _display_errors(item)
          end
        else
          puts "Item couldn't be updated:"
          puts "- #{field} isn't a valid field"
        end
      end
    end

    def done(id, weight_completed = nil)
      if item = _find_item(id)
        if weight_completed
          if item.update_attributes(weight_completed: weight_completed)
            puts "Item ##{item.id} updated."
            puts
            list_progress
          else
            puts "Item couldn't be updated:"
            _display_errors(item)
          end
        else
          item.done!
          puts "Item ##{item.id} marked as done."
          puts
          list_progress
        end
      end
    end

    def not_done(id)
      if item = _find_item(id)
        item.not_done!
        puts "Item ##{item.id} marked as not done."
        puts
        list_progress
      end
    end

    def archive(id)
      if item = _find_item(id)
        item.archive!
        puts "Archived item ##{item.id}."
        puts
        list_progress
      end
    end

    def unarchive(id)
      if item = _find_item(id)
        item.unarchive!
        puts "Unarchived item ##{item.id}."
        puts
        list_progress
      end
    end

    def delete(id)
      if item = _find_item(id)
        item.destroy
        puts "Deleted item ##{item.id}."
        puts
        list_progress
      end
    end

    def list(*flags)
      items = Item.latest
      if flags.include? "archived"
        items = Item.archived
      elsif not flags.include? "all"
        items = Item.active
      end

      table = Terminal::Table.new(title: "Todo List", headings: %w(ID Progress Item)) do |t|
        items.each do |item|
          progress = "%d / %d" % [item.weight_completed, item.weight]
          progress_bar = _render_progress_bar(item.percent_done, 10)
          archived = item.archived? ? "[A] " : ""
          t.add_row [item.id, progress, "#{progress_bar}  #{archived}#{item.title}"]
        end
      end
      puts table
    end

    def progress
      if Item.active.empty?
        puts "No active items."
      else
        total_weight = Item.active.sum(:weight)
        total_weight_completed = Item.active.sum(:weight_completed)
        percent = total_weight_completed * 100 / total_weight
        puts "%s %s%%" % [_render_progress_bar(percent, 50), percent]
      end
    end

    def list_progress
      if Item.active.empty?
        puts "There are no active items to list."
      else
        list
        puts
        progress
      end
    end

    def help
      puts "Usage: goldie [command] [arg1] [arg2] ..."
      puts
      puts "Available commands:"
      puts
      COMMAND_LIST.each do |command, summary|
        puts "  % -16s    %s" % [command, summary]
      end
    end

    def _find_item(id)
      if item = Item.where(id: id).first
        item
      else
        puts "Couldn't find item with id '#{id}'."
      end
    end

    def _display_errors(item)
      item.errors.messages.each do |field, messages|
        messages.each do |message|
          puts "- #{field} #{message}"
        end
      end
    end

    def _render_progress_bar(percent, width)
      progress = (percent * width / 100).to_i
      ("\u2589" * progress).colored.green.bold + ("\u2589" * (width - progress)).colored.white.bold
    end
  end
end
